@@ -25,18 +25,20 @@ import (
 	"net/http/httputil"
 	"net/url"
 	"os"
+	"path/filepath"
 	"strconv"
 	"time"
 
 	"github.com/kelseyhightower/envconfig"
 	"go.opencensus.io/plugin/ochttp"
-	"go.opencensus.io/stats/view"
 	"go.opencensus.io/trace"
 	"go.uber.org/zap"
 
 	"k8s.io/apimachinery/pkg/types"
 
+	network "knative.dev/networking/pkg"
 	"knative.dev/networking/pkg/apis/networking"
+	"knative.dev/pkg/injection/sharedmain"
 	pkglogging "knative.dev/pkg/logging"
 	"knative.dev/pkg/logging/logkey"
 	"knative.dev/pkg/metrics"
@@ -46,12 +48,10 @@ import (
 	"knative.dev/pkg/tracing"
 	tracingconfig "knative.dev/pkg/tracing/config"
 	"knative.dev/pkg/tracing/propagation/tracecontextb3"
-	"knative.dev/serving/pkg/activator"
 	activatorutil "knative.dev/serving/pkg/activator/util"
 	pkghttp "knative.dev/serving/pkg/http"
 	"knative.dev/serving/pkg/http/handler"
 	"knative.dev/serving/pkg/logging"
-	"knative.dev/serving/pkg/network"
 	"knative.dev/serving/pkg/queue"
 	"knative.dev/serving/pkg/queue/health"
 	"knative.dev/serving/pkg/queue/readiness"
@@ -65,9 +65,8 @@ const (
 )
 
 var (
-	logger *zap.SugaredLogger
-
 	readinessProbeTimeout = flag.Duration("probe-period", -1, "run readiness probe with given timeout")
+	unixSocketPath        = filepath.Join(os.TempDir(), "queue.sock")
 )
 
 type config struct {
@@ -82,6 +81,7 @@ type config struct {
 	ServingLoggingConfig         string `split_words:"true" required:"true"`
 	ServingLoggingLevel          string `split_words:"true" required:"true"`
 	ServingRequestLogTemplate    string `split_words:"true"` // optional
+	ServingEnableRequestLog      bool   `split_words:"true"` // optional
 	ServingEnableProbeRequestLog bool   `split_words:"true"` // optional
 
 	// Metrics configuration
@@ -92,6 +92,7 @@ type config struct {
 	ServingPod                   string `split_words:"true" required:"true"`
 	ServingService               string `split_words:"true"` // optional
 	ServingRequestMetricsBackend string `split_words:"true"` // optional
+	MetricsCollectorAddress      string `split_words:"true"` // optional
 
 	// Tracing configuration
 	TracingConfigDebug                bool                      `split_words:"true"` // optional
@@ -101,56 +102,7 @@ type config struct {
 	TracingConfigStackdriverProjectID string                    `split_words:"true"` // optional
 }
 
-// Make handler a closure for testing.
-func proxyHandler(breaker *queue.Breaker, stats *network.RequestStats, tracingEnabled bool, next http.Handler) http.HandlerFunc {
-	return func(w http.ResponseWriter, r *http.Request) {
-		if network.IsKubeletProbe(r) {
-			next.ServeHTTP(w, r)
-			return
-		}
-
-		if tracingEnabled {
-			proxyCtx, proxySpan := trace.StartSpan(r.Context(), "queue_proxy")
-			r = r.WithContext(proxyCtx)
-			defer proxySpan.End()
-		}
-
-		// Metrics for autoscaling.
-		in, out := network.ReqIn, network.ReqOut
-		if activator.Name == network.KnativeProxyHeader(r) {
-			in, out = network.ProxiedIn, network.ProxiedOut
-		}
-		stats.HandleEvent(network.ReqEvent{Time: time.Now(), Type: in})
-		defer func() {
-			stats.HandleEvent(network.ReqEvent{Time: time.Now(), Type: out})
-		}()
-		network.RewriteHostOut(r)
-
-		// Enforce queuing and concurrency limits.
-		if breaker != nil {
-			var waitSpan *trace.Span
-			if tracingEnabled {
-				_, waitSpan = trace.StartSpan(r.Context(), "queue_wait")
-			}
-			if err := breaker.Maybe(r.Context(), func() {
-				waitSpan.End()
-				next.ServeHTTP(w, r)
-			}); err != nil {
-				waitSpan.End()
-				switch err {
-				case context.DeadlineExceeded, queue.ErrRequestQueueFull:
-					http.Error(w, err.Error(), http.StatusServiceUnavailable)
-				default:
-					w.WriteHeader(http.StatusInternalServerError)
-				}
-			}
-		} else {
-			next.ServeHTTP(w, r)
-		}
-	}
-}
-
-func knativeProbeHandler(healthState *health.State, prober func() bool, isAggressive bool, tracingEnabled bool, next http.Handler, logger *zap.SugaredLogger) http.HandlerFunc {
+func knativeProbeHandler(healthState *health.State, prober func() bool, isAggressive bool, tracingEnabled bool, next http.Handler) http.HandlerFunc {
 	return func(w http.ResponseWriter, r *http.Request) {
 		ph := network.KnativeProbeHeader(r)
 
@@ -195,9 +147,19 @@ func main() {
 
 	// If this is set, we run as a standalone binary to probe the queue-proxy.
 	if *readinessProbeTimeout >= 0 {
-		os.Exit(standaloneProbeMain(*readinessProbeTimeout))
+		// Use a unix socket rather than TCP to avoid going via entire TCP stack
+		// when we're actually in the same container.
+		transport := http.DefaultTransport.(*http.Transport).Clone()
+		transport.DialContext = func(_ context.Context, _, _ string) (net.Conn, error) {
+			return net.Dial("unix", unixSocketPath)
+		}
+
+		os.Exit(standaloneProbeMain(*readinessProbeTimeout, transport))
 	}
 
+	// Otherwise, we run as the queue-proxy service.
+	ctx := signals.NewContext()
+
 	// Parse the environment.
 	var env config
 	if err := envconfig.Process("", &env); err != nil {
@@ -206,11 +168,10 @@ func main() {
 	}
 
 	// Setup the logger.
-	logger, _ = pkglogging.NewLogger(env.ServingLoggingConfig, env.ServingLoggingLevel)
-	logger = logger.Named("queueproxy")
+	logger, _ := pkglogging.NewLogger(env.ServingLoggingConfig, env.ServingLoggingLevel)
 	defer flush(logger)
 
-	logger = logger.With(
+	logger = logger.Named("queueproxy").With(
 		zap.Object(logkey.Key, pkglogging.NamespacedName(types.NamespacedName{
 			Namespace: env.ServingNamespace,
 			Name:      env.ServingRevision,
@@ -218,11 +179,7 @@ func main() {
 		zap.String(logkey.Pod, env.ServingPod))
 
 	// Report stats on Go memory usage every 30 seconds.
-	msp := metrics.NewMemStatsAll()
-	msp.Start(context.Background(), 30*time.Second)
-	if err := view.Register(msp.DefaultViews()...); err != nil {
-		logger.Fatalw("Error exporting go memstats view", zap.Error(err))
-	}
+	sharedmain.MemStatsOrDie(ctx)
 
 	// Setup reporters and processes to handle stat reporting.
 	promStatReporter, err := queue.NewPrometheusStatsReporter(
@@ -232,44 +189,75 @@ func main() {
 		logger.Fatalw("Failed to create stats reporter", zap.Error(err))
 	}
 
+	protoStatReporter := queue.NewProtobufStatsReporter(env.ServingPod, reportingPeriod)
+
 	reportTicker := time.NewTicker(reportingPeriod)
 	defer reportTicker.Stop()
 
 	stats := network.NewRequestStats(time.Now())
 	go func() {
 		for now := range reportTicker.C {
-			promStatReporter.Report(stats.Report(now))
+			stat := stats.Report(now)
+			promStatReporter.Report(stat)
+			protoStatReporter.Report(stat)
 		}
 	}()
 
 	// Setup probe to run for checking user-application healthiness.
-	probe := buildProbe(env.ServingReadinessProbe)
+	probe := buildProbe(logger, env.ServingReadinessProbe)
 	healthState := &health.State{}
 
-	server := buildServer(env, healthState, probe, stats, logger)
-	adminServer := buildAdminServer(healthState, logger)
-	metricsServer := buildMetricsServer(promStatReporter)
-
+	mainServer := buildServer(ctx, env, healthState, probe, stats, logger)
 	servers := map[string]*http.Server{
-		"main":    server,
-		"admin":   adminServer,
-		"metrics": metricsServer,
+		"main":    mainServer,
+		"admin":   buildAdminServer(logger, healthState),
+		"metrics": buildMetricsServer(promStatReporter, protoStatReporter),
 	}
-
 	if env.EnableProfiling {
 		servers["profile"] = profiling.NewServer(profiling.NewHandler(logger, true))
 	}
 
-	errCh := make(chan error, len(servers))
+	errCh := make(chan error)
+	listenCh := make(chan struct{})
 	for name, server := range servers {
 		go func(name string, s *http.Server) {
+			l, err := net.Listen("tcp", s.Addr)
+			if err != nil {
+				errCh <- fmt.Errorf("%s server failed to listen: %w", name, err)
+				return
+			}
+
+			// Notify the unix socket setup that the tcp socket for the main server is ready.
+			if s == mainServer {
+				close(listenCh)
+			}
+
 			// Don't forward ErrServerClosed as that indicates we're already shutting down.
-			if err := s.ListenAndServe(); err != nil && err != http.ErrServerClosed {
-				errCh <- fmt.Errorf("%s server failed: %w", name, err)
+			if err := s.Serve(l); err != nil && err != http.ErrServerClosed {
+				errCh <- fmt.Errorf("%s server failed to serve: %w", name, err)
 			}
 		}(name, server)
 	}
 
+	// Listen on a unix socket so that the exec probe can avoid having to go
+	// through the full tcp network stack.
+	go func() {
+		// Only start listening on the unix socket once the tcp socket for the
+		// main server is setup.
+		// This avoids the unix socket path succeeding before the tcp socket path
+		// is actually working and thus it avoids a race.
+		<-listenCh
+
+		l, err := net.Listen("unix", unixSocketPath)
+		if err != nil {
+			errCh <- fmt.Errorf("failed to listen to unix socket: %w", err)
+			return
+		}
+		if err := http.Serve(l, mainServer.Handler); err != nil {
+			errCh <- fmt.Errorf("serving failed on unix socket: %w", err)
+		}
+	}()
+
 	// Blocks until we actually receive a TERM signal or one of the servers
 	// exit unexpectedly. We fold both signals together because we only want
 	// to act on the first of those to reach here.
@@ -279,7 +267,7 @@ func main() {
 		// This extra flush is needed because defers are not handled via os.Exit calls.
 		flush(logger)
 		os.Exit(1)
-	case <-signals.SetupSignalHandler():
+	case <-ctx.Done():
 		logger.Info("Received TERM signal, attempting to gracefully shutdown servers.")
 		healthState.Shutdown(func() {
 			logger.Infof("Sleeping %v to allow K8s propagation of non-ready state", pkgnet.DefaultDrainTimeout)
@@ -288,7 +276,7 @@ func main() {
 			// Calling server.Shutdown() allows pending requests to
 			// complete, while no new work is accepted.
 			logger.Info("Shutting down main server")
-			if err := server.Shutdown(context.Background()); err != nil {
+			if err := mainServer.Shutdown(context.Background()); err != nil {
 				logger.Errorw("Failed to shutdown proxy server", zap.Error(err))
 			}
 			// Removing the main server from the shutdown logic as we've already shut it down.
@@ -305,7 +293,7 @@ func main() {
 	}
 }
 
-func buildProbe(probeJSON string) *readiness.Probe {
+func buildProbe(logger *zap.SugaredLogger, probeJSON string) *readiness.Probe {
 	coreProbe, err := readiness.DecodeProbe(probeJSON)
 	if err != nil {
 		logger.Fatalw("Queue container failed to parse readiness probe", zap.Error(err))
@@ -313,22 +301,27 @@ func buildProbe(probeJSON string) *readiness.Probe {
 	return readiness.NewProbe(coreProbe)
 }
 
-func buildServer(env config, healthState *health.State, rp *readiness.Probe, stats *network.RequestStats,
+func buildServer(ctx context.Context, env config, healthState *health.State, rp *readiness.Probe, stats *network.RequestStats,
 	logger *zap.SugaredLogger) *http.Server {
 	target := &url.URL{
 		Scheme: "http",
 		Host:   net.JoinHostPort("127.0.0.1", strconv.Itoa(env.UserPort)),
 	}
 
+	maxIdleConns := 1000 // TODO: somewhat arbitrary value for CC=0, needs experimental validation.
+	if env.ContainerConcurrency > 0 {
+		maxIdleConns = env.ContainerConcurrency
+	}
+
 	httpProxy := httputil.NewSingleHostReverseProxy(target)
-	httpProxy.Transport = buildTransport(env, logger)
+	httpProxy.Transport = buildTransport(env, logger, maxIdleConns)
 	httpProxy.ErrorHandler = pkgnet.ErrorHandler(logger)
 	httpProxy.BufferPool = network.NewBufferPool()
-	httpProxy.FlushInterval = -1
+	httpProxy.FlushInterval = network.FlushInterval
 	activatorutil.SetupHeaderPruning(httpProxy)
 
-	breaker := buildBreaker(env)
-	metricsSupported := supportsMetrics(env, logger)
+	breaker := buildBreaker(logger, env)
+	metricsSupported := supportsMetrics(ctx, logger, env)
 	tracingEnabled := env.TracingConfigBackend != tracingconfig.None
 	timeout := time.Duration(env.RevisionTimeoutSeconds) * time.Second
 
@@ -336,32 +329,35 @@ func buildServer(env config, healthState *health.State, rp *readiness.Probe, sta
 	// Note: innermost handlers are specified first, ie. the last handler in the chain will be executed first.
 	var composedHandler http.Handler = httpProxy
 	if metricsSupported {
-		composedHandler = requestAppMetricsHandler(composedHandler, breaker, env)
+		composedHandler = requestAppMetricsHandler(logger, composedHandler, breaker, env)
 	}
-	composedHandler = proxyHandler(breaker, stats, tracingEnabled, composedHandler)
+	composedHandler = queue.ProxyHandler(breaker, stats, tracingEnabled, composedHandler)
 	composedHandler = queue.ForwardedShimHandler(composedHandler)
 	composedHandler = handler.NewTimeToFirstByteTimeoutHandler(composedHandler, "request timeout", handler.StaticTimeoutFunc(timeout))
 
 	if metricsSupported {
-		composedHandler = requestMetricsHandler(composedHandler, env)
+		composedHandler = requestMetricsHandler(logger, composedHandler, env)
 	}
 	composedHandler = tracing.HTTPSpanMiddleware(composedHandler)
 
-	composedHandler = knativeProbeHandler(healthState, rp.ProbeContainer, rp.IsAggressive(), tracingEnabled, composedHandler, logger)
+	composedHandler = knativeProbeHandler(healthState, rp.ProbeContainer, rp.IsAggressive(), tracingEnabled, composedHandler)
 	composedHandler = network.NewProbeHandler(composedHandler)
 	// We might want sometimes capture the probes/healthchecks in the request
 	// logs. Hence we need to have RequestLogHandler to be the first one.
-	composedHandler = pushRequestLogHandler(composedHandler, env)
+	composedHandler = pushRequestLogHandler(logger, composedHandler, env)
 
 	return pkgnet.NewServer(":"+strconv.Itoa(env.QueueServingPort), composedHandler)
 }
 
-func buildTransport(env config, logger *zap.SugaredLogger) http.RoundTripper {
+func buildTransport(env config, logger *zap.SugaredLogger, maxConns int) http.RoundTripper {
+	// set max-idle and max-idle-per-host to same value since we're always proxying to the same host.
+	transport := pkgnet.NewAutoTransport(maxConns /* max-idle */, maxConns /* max-idle-per-host */)
+
 	if env.TracingConfigBackend == tracingconfig.None {
-		return pkgnet.AutoTransport
+		return transport
 	}
 
-	oct := tracing.NewOpenCensusTracer(tracing.WithExporter(env.ServingPod, logger))
+	oct := tracing.NewOpenCensusTracer(tracing.WithExporterFull(env.ServingPod, env.ServingPodIP, logger))
 	oct.ApplyConfig(&tracingconfig.Config{
 		Backend:              env.TracingConfigBackend,
 		Debug:                env.TracingConfigDebug,
@@ -371,12 +367,12 @@ func buildTransport(env config, logger *zap.SugaredLogger) http.RoundTripper {
 	})
 
 	return &ochttp.Transport{
-		Base:        pkgnet.AutoTransport,
+		Base:        transport,
 		Propagation: tracecontextb3.B3Egress,
 	}
 }
 
-func buildBreaker(env config) *queue.Breaker {
+func buildBreaker(logger *zap.SugaredLogger, env config) *queue.Breaker {
 	if env.ContainerConcurrency < 1 {
 		return nil
 	}
@@ -390,13 +386,13 @@ func buildBreaker(env config) *queue.Breaker {
 	return queue.NewBreaker(params)
 }
 
-func supportsMetrics(env config, logger *zap.SugaredLogger) bool {
+func supportsMetrics(ctx context.Context, logger *zap.SugaredLogger, env config) bool {
 	// Setup request metrics reporting for end-user metrics.
 	if env.ServingRequestMetricsBackend == "" {
 		return false
 	}
 
-	if err := setupMetricsExporter(env.ServingRequestMetricsBackend); err != nil {
+	if err := setupMetricsExporter(ctx, logger, env.ServingRequestMetricsBackend, env.MetricsCollectorAddress); err != nil {
 		logger.Errorw("Error setting up request metrics exporter. Request metrics will be unavailable.", zap.Error(err))
 		return false
 	}
@@ -404,7 +400,7 @@ func supportsMetrics(env config, logger *zap.SugaredLogger) bool {
 	return true
 }
 
-func buildAdminServer(healthState *health.State, logger *zap.SugaredLogger) *http.Server {
+func buildAdminServer(logger *zap.SugaredLogger, healthState *health.State) *http.Server {
 	adminMux := http.NewServeMux()
 	drainHandler := healthState.DrainHandlerFunc()
 	adminMux.HandleFunc(queue.RequestQueueDrainPath, func(w http.ResponseWriter, r *http.Request) {
@@ -418,17 +414,17 @@ func buildAdminServer(healthState *health.State, logger *zap.SugaredLogger) *htt
 	}
 }
 
-func buildMetricsServer(promStatReporter *queue.PrometheusStatsReporter) *http.Server {
+func buildMetricsServer(promStatReporter *queue.PrometheusStatsReporter, protobufStatReporter *queue.ProtobufStatsReporter) *http.Server {
 	metricsMux := http.NewServeMux()
-	metricsMux.Handle("/metrics", promStatReporter.Handler())
+	metricsMux.Handle("/metrics", queue.NewStatsHandler(promStatReporter, protobufStatReporter))
 	return &http.Server{
 		Addr:    ":" + strconv.Itoa(networking.AutoscalingQueueMetricsPort),
 		Handler: metricsMux,
 	}
 }
 
-func pushRequestLogHandler(currentHandler http.Handler, env config) http.Handler {
-	if env.ServingRequestLogTemplate == "" {
+func pushRequestLogHandler(logger *zap.SugaredLogger, currentHandler http.Handler, env config) http.Handler {
+	if !env.ServingEnableRequestLog {
 		return currentHandler
 	}
 
@@ -449,7 +445,7 @@ func pushRequestLogHandler(currentHandler http.Handler, env config) http.Handler
 	return handler
 }
 
-func requestMetricsHandler(currentHandler http.Handler, env config) http.Handler {
+func requestMetricsHandler(logger *zap.SugaredLogger, currentHandler http.Handler, env config) http.Handler {
 	h, err := queue.NewRequestMetricsHandler(currentHandler, env.ServingNamespace,
 		env.ServingService, env.ServingConfiguration, env.ServingRevision, env.ServingPod)
 	if err != nil {
@@ -459,7 +455,7 @@ func requestMetricsHandler(currentHandler http.Handler, env config) http.Handler
 	return h
 }
 
-func requestAppMetricsHandler(currentHandler http.Handler, breaker *queue.Breaker, env config) http.Handler {
+func requestAppMetricsHandler(logger *zap.SugaredLogger, currentHandler http.Handler, breaker *queue.Breaker, env config) http.Handler {
 	h, err := queue.NewAppRequestMetricsHandler(currentHandler, breaker, env.ServingNamespace,
 		env.ServingService, env.ServingConfiguration, env.ServingRevision, env.ServingPod)
 	if err != nil {
@@ -469,7 +465,7 @@ func requestAppMetricsHandler(currentHandler http.Handler, breaker *queue.Breake
 	return h
 }
 
-func setupMetricsExporter(backend string) error {
+func setupMetricsExporter(ctx context.Context, logger *zap.SugaredLogger, backend string, collectorAddress string) error {
 	// Set up OpenCensus exporter.
 	// NOTE: We use revision as the component instead of queue because queue is
 	// implementation specific. The current metrics are request relative. Using
@@ -482,9 +478,10 @@ func setupMetricsExporter(backend string) error {
 		PrometheusPort: networking.UserQueueMetricsPort,
 		ConfigMap: map[string]string{
 			metrics.BackendDestinationKey: backend,
+			"metrics.opencensus-address":  collectorAddress,
 		},
 	}
-	return metrics.UpdateExporter(ops, logger)
+	return metrics.UpdateExporter(ctx, ops, logger)
 }
 
 func flush(logger *zap.SugaredLogger) {
