@@ -12,7 +12,7 @@ function resolve_resources(){
 
   > "$resolved_file_name"
 
-  for yaml in "$dir"/*.yaml; do
+  for yaml in `find $dir -name "*.yaml" | sort`; do
     resolve_file "$yaml" "$resolved_file_name" "$image_prefix" "$image_tag"
   done
 }
@@ -41,11 +41,7 @@ function resolve_file() {
   echo "---" >> "$to"
   # 1. Rewrite image references
   # 2. Update config map entry
-  # 3. Remove comment lines
-  # 4. Remove empty lines
   sed -e "s+\(.* image: \)\(knative.dev\)\(.*/\)\(.*\)+\1${image_prefix}\4${image_tag}+g" \
       -e "s+\(.* queueSidecarImage: \)\(knative.dev\)\(.*/\)\(.*\)+\1${image_prefix}\4${image_tag}+g" \
-      -e '/^[ \t]*#/d' \
-      -e '/^[ \t]*$/d' \
       "$file" >> "$to"
 }
