@@ -69,7 +69,10 @@ function knative_setup() {
 
 # Script entry point.
 
-initialize $@ --skip-istio-addon
+# Skip installing istio as an add-on.
+# Temporarily increasing the cluster size for serving tests to rule out
+# resource/eviction as causes of flakiness.
+initialize "$@" --skip-istio-addon --min-nodes=4 --max-nodes=4
 
 # We haven't configured these deployments for high-availability,
 # so disable the chaos duck.
